@@ -1,13 +1,3 @@
-// Copyright 2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 /*!
 Defines a high-level intermediate representation for regular expressions.
 */
@@ -15,13 +5,15 @@ use std::char;
 use std::cmp;
 use std::error;
 use std::fmt;
+use std::result;
 use std::u8;
 
 use ast::Span;
 use hir::interval::{Interval, IntervalSet, IntervalSetIter};
 use unicode;
 
-pub use hir::visitor::{Visitor, visit};
+pub use hir::visitor::{visit, Visitor};
+pub use unicode::CaseFoldError;
 
 mod interval;
 pub mod literal;
@@ -75,6 +67,14 @@ pub enum ErrorKind {
     /// This occurs when an unrecognized Unicode property value could not
     /// be found.
     UnicodePropertyValueNotFound,
+    /// This occurs when a Unicode-aware Perl character class (`\w`, `\s` or
+    /// `\d`) could not be found. This can occur when the `unicode-perl`
+    /// crate feature is not enabled.
+    UnicodePerlClassNotFound,
+    /// This occurs when the Unicode simple case mapping tables are not
+    /// available, and the regular expression required Unicode aware case
+    /// insensitivity.
+    UnicodeCaseUnavailable,
     /// This occurs when the translator attempts to construct a character class
     /// that is empty.
     ///
@@ -91,6 +91,8 @@ pub enum ErrorKind {
 }
 
 impl ErrorKind {
+    // TODO: Remove this method entirely on the next breaking semver release.
+    #[allow(deprecated)]
     fn description(&self) -> &str {
         use self::ErrorKind::*;
         match *self {
@@ -98,13 +100,23 @@ impl ErrorKind {
             InvalidUtf8 => "pattern can match invalid UTF-8",
             UnicodePropertyNotFound => "Unicode property not found",
             UnicodePropertyValueNotFound => "Unicode property value not found",
+            UnicodePerlClassNotFound => {
+                "Unicode-aware Perl class not found \
+                 (make sure the unicode-perl feature is enabled)"
+            }
+            UnicodeCaseUnavailable => {
+                "Unicode-aware case insensitivity matching is not available \
+                 (make sure the unicode-case feature is enabled)"
+            }
             EmptyClassNotAllowed => "empty character classes are not allowed",
-            _ => unreachable!(),
+            __Nonexhaustive => unreachable!(),
         }
     }
 }
 
 impl error::Error for Error {
+    // TODO: Remove this method entirely on the next breaking semver release.
+    #[allow(deprecated)]
     fn description(&self) -> &str {
         self.kind.description()
     }
@@ -118,6 +130,8 @@ impl fmt::Display for Error {
 
 impl fmt::Display for ErrorKind {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        // TODO: Remove this on the next breaking semver release.
+        #[allow(deprecated)]
         f.write_str(self.description())
     }
 }
@@ -227,10 +241,9 @@ impl Hir {
         info.set_any_anchored_start(false);
         info.set_any_anchored_end(false);
         info.set_match_empty(true);
-        Hir {
-            kind: HirKind::Empty,
-            info: info,
-        }
+        info.set_literal(true);
+        info.set_alternation_literal(true);
+        Hir { kind: HirKind::Empty, info: info }
     }
 
     /// Creates a literal HIR expression.
@@ -253,10 +266,9 @@ impl Hir {
         info.set_any_anchored_start(false);
         info.set_any_anchored_end(false);
         info.set_match_empty(false);
-        Hir {
-            kind: HirKind::Literal(lit),
-            info: info,
-        }
+        info.set_literal(true);
+        info.set_alternation_literal(true);
+        Hir { kind: HirKind::Literal(lit), info: info }
     }
 
     /// Creates a class HIR expression.
@@ -271,10 +283,9 @@ impl Hir {
         info.set_any_anchored_start(false);
         info.set_any_anchored_end(false);
         info.set_match_empty(false);
-        Hir {
-            kind: HirKind::Class(class),
-            info: info,
-        }
+        info.set_literal(false);
+        info.set_alternation_literal(false);
+        Hir { kind: HirKind::Class(class), info: info }
     }
 
     /// Creates an anchor assertion HIR expression.
@@ -289,6 +300,8 @@ impl Hir {
         info.set_any_anchored_start(false);
         info.set_any_anchored_end(false);
         info.set_match_empty(true);
+        info.set_literal(false);
+        info.set_alternation_literal(false);
         if let Anchor::StartText = anchor {
             info.set_anchored_start(true);
             info.set_line_anchored_start(true);
@@ -305,10 +318,7 @@ impl Hir {
         if let Anchor::EndLine = anchor {
             info.set_line_anchored_end(true);
         }
-        Hir {
-            kind: HirKind::Anchor(anchor),
-            info: info,
-        }
+        Hir { kind: HirKind::Anchor(anchor), info: info }
     }
 
     /// Creates a word boundary assertion HIR expression.
@@ -322,6 +332,8 @@ impl Hir {
         info.set_line_anchored_end(false);
         info.set_any_anchored_start(false);
         info.set_any_anchored_end(false);
+        info.set_literal(false);
+        info.set_alternation_literal(false);
         // A negated word boundary matches the empty string, but a normal
         // word boundary does not!
         info.set_match_empty(word_boundary.is_negated());
@@ -329,10 +341,7 @@ impl Hir {
         if let WordBoundary::AsciiNegate = word_boundary {
             info.set_always_utf8(false);
         }
-        Hir {
-            kind: HirKind::WordBoundary(word_boundary),
-            info: info,
-        }
+        Hir { kind: HirKind::WordBoundary(word_boundary), info: info }
     }
 
     /// Creates a repetition HIR expression.
@@ -343,24 +352,23 @@ impl Hir {
         // If this operator can match the empty string, then it can never
         // be anchored.
         info.set_anchored_start(
-            !rep.is_match_empty() && rep.hir.is_anchored_start()
+            !rep.is_match_empty() && rep.hir.is_anchored_start(),
         );
         info.set_anchored_end(
-            !rep.is_match_empty() && rep.hir.is_anchored_end()
+            !rep.is_match_empty() && rep.hir.is_anchored_end(),
         );
         info.set_line_anchored_start(
-            !rep.is_match_empty() && rep.hir.is_anchored_start()
+            !rep.is_match_empty() && rep.hir.is_anchored_start(),
         );
         info.set_line_anchored_end(
-            !rep.is_match_empty() && rep.hir.is_anchored_end()
+            !rep.is_match_empty() && rep.hir.is_anchored_end(),
         );
         info.set_any_anchored_start(rep.hir.is_any_anchored_start());
         info.set_any_anchored_end(rep.hir.is_any_anchored_end());
         info.set_match_empty(rep.is_match_empty() || rep.hir.is_match_empty());
-        Hir {
-            kind: HirKind::Repetition(rep),
-            info: info,
-        }
+        info.set_literal(false);
+        info.set_alternation_literal(false);
+        Hir { kind: HirKind::Repetition(rep), info: info }
     }
 
     /// Creates a group HIR expression.
@@ -375,10 +383,9 @@ impl Hir {
         info.set_any_anchored_start(group.hir.is_any_anchored_start());
         info.set_any_anchored_end(group.hir.is_any_anchored_end());
         info.set_match_empty(group.hir.is_match_empty());
-        Hir {
-            kind: HirKind::Group(group),
-            info: info,
-        }
+        info.set_literal(false);
+        info.set_alternation_literal(false);
+        Hir { kind: HirKind::Group(group), info: info }
     }
 
     /// Returns the concatenation of the given expressions.
@@ -387,7 +394,7 @@ impl Hir {
     pub fn concat(mut exprs: Vec<Hir>) -> Hir {
         match exprs.len() {
             0 => Hir::empty(),
-            1 => { exprs.pop().unwrap() }
+            1 => exprs.pop().unwrap(),
             _ => {
                 let mut info = HirInfo::new();
                 info.set_always_utf8(true);
@@ -395,6 +402,8 @@ impl Hir {
                 info.set_any_anchored_start(false);
                 info.set_any_anchored_end(false);
                 info.set_match_empty(true);
+                info.set_literal(true);
+                info.set_alternation_literal(true);
 
                 // Some attributes require analyzing all sub-expressions.
                 for e in &exprs {
@@ -404,18 +413,23 @@ impl Hir {
                     let x = info.is_all_assertions() && e.is_all_assertions();
                     info.set_all_assertions(x);
 
-                    let x =
-                        info.is_any_anchored_start()
+                    let x = info.is_any_anchored_start()
                         || e.is_any_anchored_start();
                     info.set_any_anchored_start(x);
 
                     let x =
-                        info.is_any_anchored_end()
-                        || e.is_any_anchored_end();
+                        info.is_any_anchored_end() || e.is_any_anchored_end();
                     info.set_any_anchored_end(x);
 
                     let x = info.is_match_empty() && e.is_match_empty();
                     info.set_match_empty(x);
+
+                    let x = info.is_literal() && e.is_literal();
+                    info.set_literal(x);
+
+                    let x = info.is_alternation_literal()
+                        && e.is_alternation_literal();
+                    info.set_alternation_literal(x);
                 }
                 // Anchored attributes require something slightly more
                 // sophisticated. Normally, WLOG, to determine whether an
@@ -427,45 +441,42 @@ impl Hir {
                 // is actually one that is either not an assertion or is
                 // specifically the StartText assertion.
                 info.set_anchored_start(
-                    exprs.iter()
+                    exprs
+                        .iter()
                         .take_while(|e| {
                             e.is_anchored_start() || e.is_all_assertions()
                         })
-                        .any(|e| {
-                            e.is_anchored_start()
-                        }));
+                        .any(|e| e.is_anchored_start()),
+                );
                 // Similarly for the end anchor, but in reverse.
                 info.set_anchored_end(
-                    exprs.iter()
+                    exprs
+                        .iter()
                         .rev()
                         .take_while(|e| {
                             e.is_anchored_end() || e.is_all_assertions()
                         })
-                        .any(|e| {
-                            e.is_anchored_end()
-                        }));
+                        .any(|e| e.is_anchored_end()),
+                );
                 // Repeat the process for line anchors.
                 info.set_line_anchored_start(
-                    exprs.iter()
+                    exprs
+                        .iter()
                         .take_while(|e| {
                             e.is_line_anchored_start() || e.is_all_assertions()
                         })
-                        .any(|e| {
-                            e.is_line_anchored_start()
-                        }));
+                        .any(|e| e.is_line_anchored_start()),
+                );
                 info.set_line_anchored_end(
-                    exprs.iter()
+                    exprs
+                        .iter()
                         .rev()
                         .take_while(|e| {
                             e.is_line_anchored_end() || e.is_all_assertions()
                         })
-                        .any(|e| {
-                            e.is_line_anchored_end()
-                        }));
-                Hir {
-                    kind: HirKind::Concat(exprs),
-                    info: info,
-                }
+                        .any(|e| e.is_line_anchored_end()),
+                );
+                Hir { kind: HirKind::Concat(exprs), info: info }
             }
         }
     }
@@ -488,6 +499,8 @@ impl Hir {
                 info.set_any_anchored_start(false);
                 info.set_any_anchored_end(false);
                 info.set_match_empty(false);
+                info.set_literal(false);
+                info.set_alternation_literal(true);
 
                 // Some attributes require analyzing all sub-expressions.
                 for e in &exprs {
@@ -511,23 +524,21 @@ impl Hir {
                         && e.is_line_anchored_end();
                     info.set_line_anchored_end(x);
 
-                    let x =
-                        info.is_any_anchored_start()
+                    let x = info.is_any_anchored_start()
                         || e.is_any_anchored_start();
                     info.set_any_anchored_start(x);
 
                     let x =
-                        info.is_any_anchored_end()
-                        || e.is_any_anchored_end();
+                        info.is_any_anchored_end() || e.is_any_anchored_end();
                     info.set_any_anchored_end(x);
 
                     let x = info.is_match_empty() || e.is_match_empty();
                     info.set_match_empty(x);
+
+                    let x = info.is_alternation_literal() && e.is_literal();
+                    info.set_alternation_literal(x);
                 }
-                Hir {
-                    kind: HirKind::Alternation(exprs),
-                    info: info,
-                }
+                Hir { kind: HirKind::Alternation(exprs), info: info }
             }
         }
     }
@@ -655,6 +666,28 @@ impl Hir {
     pub fn is_match_empty(&self) -> bool {
         self.info.is_match_empty()
     }
+
+    /// Return true if and only if this HIR is a simple literal. This is only
+    /// true when this HIR expression is either itself a `Literal` or a
+    /// concatenation of only `Literal`s.
+    ///
+    /// For example, `f` and `foo` are literals, but `f+`, `(foo)`, `foo()`
+    /// are not (even though that contain sub-expressions that are literals).
+    pub fn is_literal(&self) -> bool {
+        self.info.is_literal()
+    }
+
+    /// Return true if and only if this HIR is either a simple literal or an
+    /// alternation of simple literals. This is only
+    /// true when this HIR expression is either itself a `Literal` or a
+    /// concatenation of only `Literal`s or an alternation of only `Literal`s.
+    ///
+    /// For example, `f`, `foo`, `a|b|c`, and `foo|bar|baz` are alternaiton
+    /// literals, but `f+`, `(foo)`, `foo()`
+    /// are not (even though that contain sub-expressions that are literals).
+    pub fn is_alternation_literal(&self) -> bool {
+        self.info.is_alternation_literal()
+    }
 }
 
 impl HirKind {
@@ -807,7 +840,8 @@ impl ClassUnicode {
     /// The given ranges do not need to be in any specific order, and ranges
     /// may overlap.
     pub fn new<I>(ranges: I) -> ClassUnicode
-    where I: IntoIterator<Item=ClassUnicodeRange>
+    where
+        I: IntoIterator<Item = ClassUnicodeRange>,
     {
         ClassUnicode { set: IntervalSet::new(ranges) }
     }
@@ -838,8 +872,38 @@ impl ClassUnicode {
     /// characters, according to Unicode's "simple" mapping. For example, if
     /// this class consists of the range `a-z`, then applying case folding will
     /// result in the class containing both the ranges `a-z` and `A-Z`.
+    ///
+    /// # Panics
+    ///
+    /// This routine panics when the case mapping data necessary for this
+    /// routine to complete is unavailable. This occurs when the `unicode-case`
+    /// feature is not enabled.
+    ///
+    /// Callers should prefer using `try_case_fold_simple` instead, which will
+    /// return an error instead of panicking.
     pub fn case_fold_simple(&mut self) {
-        self.set.case_fold_simple();
+        self.set
+            .case_fold_simple()
+            .expect("unicode-case feature must be enabled");
+    }
+
+    /// Expand this character class such that it contains all case folded
+    /// characters, according to Unicode's "simple" mapping. For example, if
+    /// this class consists of the range `a-z`, then applying case folding will
+    /// result in the class containing both the ranges `a-z` and `A-Z`.
+    ///
+    /// # Panics
+    ///
+    /// This routine panics when the case mapping data necessary for this
+    /// routine to complete is unavailable. This occurs when the `unicode-case`
+    /// feature is not enabled.
+    ///
+    /// Callers should prefer using `try_case_fold_simple` instead, which will
+    /// return an error instead of panicking.
+    pub fn try_case_fold_simple(
+        &mut self,
+    ) -> result::Result<(), CaseFoldError> {
+        self.set.case_fold_simple()
     }
 
     /// Negate this character class.
@@ -905,40 +969,54 @@ pub struct ClassUnicodeRange {
 
 impl fmt::Debug for ClassUnicodeRange {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let start =
-            if !self.start.is_whitespace() && !self.start.is_control() {
-                self.start.to_string()
-            } else {
-                format!("0x{:X}", self.start as u32)
-            };
-        let end =
-            if !self.end.is_whitespace() && !self.end.is_control() {
-                self.end.to_string()
-            } else {
-                format!("0x{:X}", self.end as u32)
-            };
+        let start = if !self.start.is_whitespace() && !self.start.is_control()
+        {
+            self.start.to_string()
+        } else {
+            format!("0x{:X}", self.start as u32)
+        };
+        let end = if !self.end.is_whitespace() && !self.end.is_control() {
+            self.end.to_string()
+        } else {
+            format!("0x{:X}", self.end as u32)
+        };
         f.debug_struct("ClassUnicodeRange")
-         .field("start", &start)
-         .field("end", &end)
-         .finish()
+            .field("start", &start)
+            .field("end", &end)
+            .finish()
     }
 }
 
 impl Interval for ClassUnicodeRange {
     type Bound = char;
 
-    #[inline] fn lower(&self) -> char { self.start }
-    #[inline] fn upper(&self) -> char { self.end }
-    #[inline] fn set_lower(&mut self, bound: char) { self.start = bound; }
-    #[inline] fn set_upper(&mut self, bound: char) { self.end = bound; }
+    #[inline]
+    fn lower(&self) -> char {
+        self.start
+    }
+    #[inline]
+    fn upper(&self) -> char {
+        self.end
+    }
+    #[inline]
+    fn set_lower(&mut self, bound: char) {
+        self.start = bound;
+    }
+    #[inline]
+    fn set_upper(&mut self, bound: char) {
+        self.end = bound;
+    }
 
     /// Apply simple case folding to this Unicode scalar value range.
     ///
     /// Additional ranges are appended to the given vector. Canonical ordering
     /// is *not* maintained in the given vector.
-    fn case_fold_simple(&self, ranges: &mut Vec<ClassUnicodeRange>) {
-        if !unicode::contains_simple_case_mapping(self.start, self.end) {
-            return;
+    fn case_fold_simple(
+        &self,
+        ranges: &mut Vec<ClassUnicodeRange>,
+    ) -> Result<(), unicode::CaseFoldError> {
+        if !unicode::contains_simple_case_mapping(self.start, self.end)? {
+            return Ok(());
         }
         let start = self.start as u32;
         let end = (self.end as u32).saturating_add(1);
@@ -947,7 +1025,7 @@ impl Interval for ClassUnicodeRange {
             if next_simple_cp.map_or(false, |next| cp < next) {
                 continue;
             }
-            let it = match unicode::simple_fold(cp) {
+            let it = match unicode::simple_fold(cp)? {
                 Ok(it) => it,
                 Err(next) => {
                     next_simple_cp = next;
@@ -958,6 +1036,7 @@ impl Interval for ClassUnicodeRange {
                 ranges.push(ClassUnicodeRange::new(cp_folded, cp_folded));
             }
         }
+        Ok(())
     }
 }
 
@@ -1000,7 +1079,8 @@ impl ClassBytes {
     /// The given ranges do not need to be in any specific order, and ranges
     /// may overlap.
     pub fn new<I>(ranges: I) -> ClassBytes
-    where I: IntoIterator<Item=ClassBytesRange>
+    where
+        I: IntoIterator<Item = ClassBytesRange>,
     {
         ClassBytes { set: IntervalSet::new(ranges) }
     }
@@ -1035,7 +1115,7 @@ impl ClassBytes {
     /// Note that this only applies ASCII case folding, which is limited to the
     /// characters `a-z` and `A-Z`.
     pub fn case_fold_simple(&mut self) {
-        self.set.case_fold_simple();
+        self.set.case_fold_simple().expect("ASCII case folding never fails");
     }
 
     /// Negate this byte class.
@@ -1107,17 +1187,32 @@ pub struct ClassBytesRange {
 impl Interval for ClassBytesRange {
     type Bound = u8;
 
-    #[inline] fn lower(&self) -> u8 { self.start }
-    #[inline] fn upper(&self) -> u8 { self.end }
-    #[inline] fn set_lower(&mut self, bound: u8) { self.start = bound; }
-    #[inline] fn set_upper(&mut self, bound: u8) { self.end = bound; }
+    #[inline]
+    fn lower(&self) -> u8 {
+        self.start
+    }
+    #[inline]
+    fn upper(&self) -> u8 {
+        self.end
+    }
+    #[inline]
+    fn set_lower(&mut self, bound: u8) {
+        self.start = bound;
+    }
+    #[inline]
+    fn set_upper(&mut self, bound: u8) {
+        self.end = bound;
+    }
 
     /// Apply simple case folding to this byte range. Only ASCII case mappings
     /// (for a-z) are applied.
     ///
     /// Additional ranges are appended to the given vector. Canonical ordering
     /// is *not* maintained in the given vector.
-    fn case_fold_simple(&self, ranges: &mut Vec<ClassBytesRange>) {
+    fn case_fold_simple(
+        &self,
+        ranges: &mut Vec<ClassBytesRange>,
+    ) -> Result<(), unicode::CaseFoldError> {
         if !ClassBytesRange::new(b'a', b'z').is_intersection_empty(self) {
             let lower = cmp::max(self.start, b'a');
             let upper = cmp::min(self.end, b'z');
@@ -1128,6 +1223,7 @@ impl Interval for ClassBytesRange {
             let upper = cmp::min(self.end, b'Z');
             ranges.push(ClassBytesRange::new(lower + 32, upper + 32));
         }
+        Ok(())
     }
 }
 
@@ -1218,8 +1314,8 @@ impl WordBoundary {
     /// Returns true if and only if this word boundary assertion is negated.
     pub fn is_negated(&self) -> bool {
         match *self {
-            WordBoundary::Unicode |  WordBoundary::Ascii => false,
-            WordBoundary::UnicodeNegate |  WordBoundary::AsciiNegate => true,
+            WordBoundary::Unicode | WordBoundary::Ascii => false,
+            WordBoundary::UnicodeNegate | WordBoundary::AsciiNegate => true,
         }
     }
 }
@@ -1401,9 +1497,7 @@ macro_rules! define_bool {
 
 impl HirInfo {
     fn new() -> HirInfo {
-        HirInfo {
-            bools: 0,
-        }
+        HirInfo { bools: 0 }
     }
 
     define_bool!(0, is_always_utf8, set_always_utf8);
@@ -1415,6 +1509,8 @@ impl HirInfo {
     define_bool!(6, is_any_anchored_start, set_any_anchored_start);
     define_bool!(7, is_any_anchored_end, set_any_anchored_end);
     define_bool!(8, is_match_empty, set_match_empty);
+    define_bool!(9, is_literal, set_literal);
+    define_bool!(10, is_alternation_literal, set_alternation_literal);
 }
 
 #[cfg(test)]
@@ -1430,10 +1526,8 @@ mod tests {
     }
 
     fn bclass(ranges: &[(u8, u8)]) -> ClassBytes {
-        let ranges: Vec<ClassBytesRange> = ranges
-            .iter()
-            .map(|&(s, e)| ClassBytesRange::new(s, e))
-            .collect();
+        let ranges: Vec<ClassBytesRange> =
+            ranges.iter().map(|&(s, e)| ClassBytesRange::new(s, e)).collect();
         ClassBytes::new(ranges)
     }
 
@@ -1441,6 +1535,7 @@ mod tests {
         cls.iter().map(|x| (x.start(), x.end())).collect()
     }
 
+    #[cfg(feature = "unicode-case")]
     fn ucasefold(cls: &ClassUnicode) -> ClassUnicode {
         let mut cls_ = cls.clone();
         cls_.case_fold_simple();
@@ -1465,7 +1560,10 @@ mod tests {
         cls_
     }
 
-    fn usymdifference(cls1: &ClassUnicode, cls2: &ClassUnicode) -> ClassUnicode {
+    fn usymdifference(
+        cls1: &ClassUnicode,
+        cls2: &ClassUnicode,
+    ) -> ClassUnicode {
         let mut cls_ = cls1.clone();
         cls_.symmetric_difference(cls2);
         cls_
@@ -1546,8 +1644,12 @@ mod tests {
         assert_eq!(expected, uranges(&cls));
 
         let cls = uclass(&[
-            ('c', 'f'), ('a', 'g'), ('d', 'j'), ('a', 'c'),
-            ('m', 'p'), ('l', 's'),
+            ('c', 'f'),
+            ('a', 'g'),
+            ('d', 'j'),
+            ('a', 'c'),
+            ('m', 'p'),
+            ('l', 's'),
         ]);
         let expected = vec![('a', 'j'), ('l', 's')];
         assert_eq!(expected, uranges(&cls));
@@ -1560,7 +1662,6 @@ mod tests {
         let expected = vec![('\x00', '\u{10FFFF}')];
         assert_eq!(expected, uranges(&cls));
 
-
         let cls = uclass(&[('a', 'a'), ('b', 'b')]);
         let expected = vec![('a', 'b')];
         assert_eq!(expected, uranges(&cls));
@@ -1581,8 +1682,12 @@ mod tests {
         assert_eq!(expected, branges(&cls));
 
         let cls = bclass(&[
-            (b'c', b'f'), (b'a', b'g'), (b'd', b'j'), (b'a', b'c'),
-            (b'm', b'p'), (b'l', b's'),
+            (b'c', b'f'),
+            (b'a', b'g'),
+            (b'd', b'j'),
+            (b'a', b'c'),
+            (b'm', b'p'),
+            (b'l', b's'),
         ]);
         let expected = vec![(b'a', b'j'), (b'l', b's')];
         assert_eq!(expected, branges(&cls));
@@ -1601,21 +1706,30 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "unicode-case")]
     fn class_case_fold_unicode() {
         let cls = uclass(&[
-            ('C', 'F'), ('A', 'G'), ('D', 'J'), ('A', 'C'),
-            ('M', 'P'), ('L', 'S'), ('c', 'f'),
+            ('C', 'F'),
+            ('A', 'G'),
+            ('D', 'J'),
+            ('A', 'C'),
+            ('M', 'P'),
+            ('L', 'S'),
+            ('c', 'f'),
         ]);
         let expected = uclass(&[
-            ('A', 'J'), ('L', 'S'),
-            ('a', 'j'), ('l', 's'),
+            ('A', 'J'),
+            ('L', 'S'),
+            ('a', 'j'),
+            ('l', 's'),
             ('\u{17F}', '\u{17F}'),
         ]);
         assert_eq!(expected, ucasefold(&cls));
 
         let cls = uclass(&[('A', 'Z')]);
         let expected = uclass(&[
-            ('A', 'Z'), ('a', 'z'),
+            ('A', 'Z'),
+            ('a', 'z'),
             ('\u{17F}', '\u{17F}'),
             ('\u{212A}', '\u{212A}'),
         ]);
@@ -1623,7 +1737,8 @@ mod tests {
 
         let cls = uclass(&[('a', 'z')]);
         let expected = uclass(&[
-            ('A', 'Z'), ('a', 'z'),
+            ('A', 'Z'),
+            ('a', 'z'),
             ('\u{17F}', '\u{17F}'),
             ('\u{212A}', '\u{212A}'),
         ]);
@@ -1641,25 +1756,58 @@ mod tests {
         assert_eq!(cls, ucasefold(&cls));
 
         let cls = uclass(&[('k', 'k')]);
-        let expected = uclass(&[
-            ('K', 'K'), ('k', 'k'), ('\u{212A}', '\u{212A}'),
-        ]);
+        let expected =
+            uclass(&[('K', 'K'), ('k', 'k'), ('\u{212A}', '\u{212A}')]);
         assert_eq!(expected, ucasefold(&cls));
 
         let cls = uclass(&[('@', '@')]);
         assert_eq!(cls, ucasefold(&cls));
     }
 
+    #[test]
+    #[cfg(not(feature = "unicode-case"))]
+    fn class_case_fold_unicode_disabled() {
+        let mut cls = uclass(&[
+            ('C', 'F'),
+            ('A', 'G'),
+            ('D', 'J'),
+            ('A', 'C'),
+            ('M', 'P'),
+            ('L', 'S'),
+            ('c', 'f'),
+        ]);
+        assert!(cls.try_case_fold_simple().is_err());
+    }
+
+    #[test]
+    #[should_panic]
+    #[cfg(not(feature = "unicode-case"))]
+    fn class_case_fold_unicode_disabled_panics() {
+        let mut cls = uclass(&[
+            ('C', 'F'),
+            ('A', 'G'),
+            ('D', 'J'),
+            ('A', 'C'),
+            ('M', 'P'),
+            ('L', 'S'),
+            ('c', 'f'),
+        ]);
+        cls.case_fold_simple();
+    }
+
     #[test]
     fn class_case_fold_bytes() {
         let cls = bclass(&[
-            (b'C', b'F'), (b'A', b'G'), (b'D', b'J'), (b'A', b'C'),
-            (b'M', b'P'), (b'L', b'S'), (b'c', b'f'),
-        ]);
-        let expected = bclass(&[
-            (b'A', b'J'), (b'L', b'S'),
-            (b'a', b'j'), (b'l', b's'),
+            (b'C', b'F'),
+            (b'A', b'G'),
+            (b'D', b'J'),
+            (b'A', b'C'),
+            (b'M', b'P'),
+            (b'L', b'S'),
+            (b'c', b'f'),
         ]);
+        let expected =
+            bclass(&[(b'A', b'J'), (b'L', b'S'), (b'a', b'j'), (b'l', b's')]);
         assert_eq!(expected, bcasefold(&cls));
 
         let cls = bclass(&[(b'A', b'Z')]);
@@ -1701,7 +1849,9 @@ mod tests {
 
         let cls = uclass(&[('a', 'c'), ('x', 'z')]);
         let expected = uclass(&[
-            ('\x00', '\x60'), ('\x64', '\x77'), ('\x7B', '\u{10FFFF}'),
+            ('\x00', '\x60'),
+            ('\x64', '\x77'),
+            ('\x7B', '\u{10FFFF}'),
         ]);
         assert_eq!(expected, unegate(&cls));
 
@@ -1721,9 +1871,8 @@ mod tests {
         let expected = uclass(&[('\x00', '\u{10FFFF}')]);
         assert_eq!(expected, unegate(&cls));
 
-        let cls = uclass(&[
-            ('\x00', '\u{10FFFD}'), ('\u{10FFFF}', '\u{10FFFF}'),
-        ]);
+        let cls =
+            uclass(&[('\x00', '\u{10FFFD}'), ('\u{10FFFF}', '\u{10FFFF}')]);
         let expected = uclass(&[('\u{10FFFE}', '\u{10FFFE}')]);
         assert_eq!(expected, unegate(&cls));
 
@@ -1756,7 +1905,9 @@ mod tests {
 
         let cls = bclass(&[(b'a', b'c'), (b'x', b'z')]);
         let expected = bclass(&[
-            (b'\x00', b'\x60'), (b'\x64', b'\x77'), (b'\x7B', b'\xFF'),
+            (b'\x00', b'\x60'),
+            (b'\x64', b'\x77'),
+            (b'\x7B', b'\xFF'),
         ]);
         assert_eq!(expected, bnegate(&cls));
 
@@ -2128,7 +2279,7 @@ mod tests {
         // We run our test on a thread with a small stack size so we can
         // force the issue more easily.
         thread::Builder::new()
-            .stack_size(1<<10)
+            .stack_size(1 << 10)
             .spawn(run)
             .unwrap()
             .join()
